`include "inc/noc.svh"
`include "inc/noc_XY.svh"
`include "src/router.sv"

module noc(
    input clk,
    output[0:`PL-1] core_inputs[0:`Y-1][0:`X-1],
    input[0:`PL-1] core_outputs[0:`Y-1][0:`X-1],
	input core_availability_signals_out[0:`Y-1][0:`X-1],
	output core_availability_signals_in[0:`Y-1][0:`X-1]
);

wire [0:`PL-1] inputs[0:`Y-1][0:`X-1][0:`REN-1];

wire availability_signals[0:`Y-1][0:`X-1][0:`REN-1];

generate
    genvar router_Y_iterator, router_X_iterator;
    for (router_Y_iterator = 0; router_Y_iterator < `Y; router_Y_iterator = router_Y_iterator + 1)
    begin  : routers_Y
        for (router_X_iterator = 0; router_X_iterator < `X; router_X_iterator = router_X_iterator + 1)
        begin  : routers_X

            assign core_availability_signals_in[router_Y_iterator][router_X_iterator] = availability_signals[router_Y_iterator][router_X_iterator][0];

            localparam lower = router_Y_iterator == `Y-1 ? 0 : router_Y_iterator + 1;
            localparam right = router_X_iterator == `X-1 ? 0 : router_X_iterator + 1;
            localparam upper = router_Y_iterator == 0 ? `Y-1 : router_Y_iterator - 1;
            localparam left = router_X_iterator == 0 ? `X-1 : router_X_iterator - 1;

            wire[0:`PL-1] outputs[0:`REN-1];
            assign core_inputs[router_Y_iterator][router_X_iterator] = outputs[0];
            assign inputs[upper][router_X_iterator][3] = outputs[1];
            assign inputs[router_Y_iterator][right][4] = outputs[2];
            assign inputs[lower][router_X_iterator][1] = outputs[3];
            assign inputs[router_Y_iterator][left][2] = outputs[4];

            wire signals_in[0:`REN-1];
            assign signals_in[0] = core_availability_signals_out[router_Y_iterator][router_X_iterator];
            assign signals_in[1] = availability_signals[upper][router_X_iterator][3];
            assign signals_in[2] = availability_signals[router_Y_iterator][right][4];
            assign signals_in[3] = availability_signals[lower][router_X_iterator][1];
            assign signals_in[4] = availability_signals[router_Y_iterator][left][2];

            router router(
                .clk(clk),
                .inputs(inputs[router_Y_iterator][router_X_iterator]), .outputs(outputs),
                .signals_out(availability_signals[router_Y_iterator][router_X_iterator]), .signals_in(signals_in),
                .router_Y(router_Y_iterator), .router_X(router_X_iterator)
            );
			
			assign inputs[router_Y_iterator][router_X_iterator][0] = core_outputs[router_Y_iterator][router_X_iterator];
        end
    end
endgenerate

always @(posedge clk)
    begin
    end

endmodule
