// This is an XY example
`include "inc/router.svh"
`include "inc/noc_CIRCULANT.svh"

module algorithm
(
    input[0:`PL-1] from_arbiter,
    input[`CS-1:0] router_no,
    input[`REN_B-1:0] shift,
    output shift_signals[0:`REN-1],
    input availability_signals_in[0:`REN-1],
    output wire[0:`PL-1] outputs[0:`REN-1]
);
    
    wire [`CS:0] router_quantity = `RN;
    wire [`CS:0] long_length = `LT;
    wire signed[`CS:0] destination = {1'b0, from_arbiter[1:`CS]};
    wire signed[`CS:0] long_length_s = {1'b0, long_length};
    wire signed[`CS:0] relative_destination;

    wire[0:1] drc_1; // destination_router_comparator_1
    assign drc_1[0] = destination > router_no;
    assign drc_1[1] = destination < router_no;

    wire[0:4] drc_2; // destination_router_comparator_2
    assign drc_2[0] = (drc_1 == 2'b00);
    assign drc_2[1] = drc_1[0] & (destination - router_no <= (router_no + router_quantity) - destination);
    assign drc_2[2] = drc_1[0] & !drc_2[1];
    assign drc_2[3] = drc_1[1] & (router_no - destination <= (destination + router_quantity) - router_no);
    assign drc_2[4] = drc_1[1] & !drc_2[3];

    assign relative_destination = drc_2[0] ? 0 :
                                    drc_2[1] ? destination - router_no :
                                    drc_2[2] ? destination - (router_no + router_quantity) :
                                    drc_2[3] ? destination - router_no :
                                    drc_2[4] ? (destination + router_quantity) - router_no :
                                    0;

    wire[0:1] selector_step1 = {relative_destination < 0, relative_destination > 0};

    wire[0:2] selector_step2;
    assign selector_step2[0] = (relative_destination > long_length_s) |
                                (-relative_destination > long_length_s);
    assign selector_step2[1] = (long_length_s - relative_destination < relative_destination) |
                                (long_length_s + relative_destination < -relative_destination);
    assign selector_step2[2] = (selector_step2[0] == 0) & (selector_step2[1] == 0);


    wire[0:4] selector;
    assign selector[0] = (selector_step1 == 2'b00) & availability_signals_in[0] & from_arbiter[0];
    assign selector[1] = selector_step1[0] & (selector_step2[0] | selector_step2[1]) & availability_signals_in[1] & from_arbiter[0];
    assign selector[2] = selector_step1[0] & selector_step2[2] & availability_signals_in[2] & from_arbiter[0];
    assign selector[3] = selector_step1[1] & selector_step2[2] & availability_signals_in[3] & from_arbiter[0];
    assign selector[4] = selector_step1[1] & (selector_step2[0] | selector_step2[1]) & availability_signals_in[4] & from_arbiter[0];


    wire shift_required = selector[0] | selector[1] | selector[2] | selector[3] | selector[4];
    
    assign shift_signals[0] = shift_required & (shift == 0);
    assign shift_signals[1] = shift_required & (shift == 1);
    assign shift_signals[2] = shift_required & (shift == 2);
    assign shift_signals[3] = shift_required & (shift == 3);
    assign shift_signals[4] = shift_required & (shift == 4);


    generate
        genvar i, j;
        for (i = 0; i < `PL; i = i + 1 ) begin : generate_outputs_bits_core
            assign outputs[0][i] = from_arbiter[i] & selector[0];
        end

        for (i = 0; i < `PL; i = i + 1 ) begin : generate_outputs_bits_left_long
            assign outputs[1][i] = from_arbiter[i] & selector[1];
        end

        for (i = 0; i < `PL; i = i + 1 ) begin : generate_outputs_bits_left_short
            assign outputs[2][i] = from_arbiter[i] & selector[2] & !selector[1] & !selector[3];
        end

        for (i = 0; i < `PL; i = i + 1 ) begin : generate_outputs_bits_right_short
            assign outputs[3][i] = from_arbiter[i] & selector[3];
        end

        for (i = 0; i < `PL; i = i + 1 ) begin : generate_outputs_bits_right_long
            assign outputs[4][i] = from_arbiter[i] & selector[4] & !selector[1] & !selector[3];
        end

    endgenerate

endmodule