`include "inc/queue.svh"

module buffer (
        input clk,
        input[0:`PL-1] data_in,
        output reg[0:`PL-1] data_out = 0,
        input shift_signal
    );

    always @(posedge clk) begin
        if (shift_signal | !data_out[0]) begin
            data_out <= data_in;
        end else begin end
    end

endmodule

module queue (
    input clk,
    input[0:`PL-1] data_in,
    output[0:`PL-1] data_out,
    input shift_signal,
    output availability_signal
);

wire[0:`PL-1] inner_data_wires[0:`EN];
wire[0:`PL-1] inner_data_wires_insertion[0:`EN];

generate
    genvar i;
    for (i = 0; i<`EN; i=i+1) begin : buffer_initialiser
        if (i == 0)
        begin
            buffer buffer_i(
            .clk(clk),
            .data_in(inner_data_wires_insertion[i]),
            .data_out(inner_data_wires[i+1]),
            .shift_signal(shift_signal)
            );
        end
        else
        begin            
            buffer buffer_i(
            .clk(clk),
            .data_in(inner_data_wires[i]|inner_data_wires_insertion[i]),
            .data_out(inner_data_wires[i+1]),
            .shift_signal(shift_signal)
            );
        end
    end
endgenerate

assign availability_signal = ~inner_data_wires[1][0];
assign data_out = inner_data_wires[`EN];

reg[`EN_B+1:0] available_buffer_pointer = `EN-1;

wire[`EN_B:0] insert_index = $signed(available_buffer_pointer+shift_signal) < `EN ? available_buffer_pointer+shift_signal : `EN-1;

generate
    for (i = 0; i<`EN; i=i+1) begin : insertion_initialiser
        assign inner_data_wires_insertion[i] = i == insert_index & availability_signal ? data_in: 0;
    end
endgenerate

always @(posedge clk) begin
    if(inner_data_wires_insertion[insert_index][0])
    begin
        available_buffer_pointer <= available_buffer_pointer - (!shift_signal | available_buffer_pointer == (`EN - 1));
    end 
    else 
    begin
        available_buffer_pointer <= insert_index;
    end
end

endmodule
