`include "noc/toplevel.sv"
`include "inc/noc.svh"
`timescale 1ns/1ns

module tb_routing;

reg clk = 0;
reg[0:`PL-1] packet = 0;
reg[`CS-1:0] core_NO = 0;
reg core_availability_signals_out[0:`RN-1];
wire core_availability_signals_in[0:`RN-1];
wire[0:1] core_inputs_2_bits[0:`RN-1];

reg[0:`PL-1] memory[0:`RN-1][0:1];

reg[31:0] i = 0;
reg[31:0] j = 0;
reg[31:0] t = 0;

toplevel DUT (clk, packet, core_NO, core_availability_signals_out, core_availability_signals_in, core_inputs_2_bits);

localparam CLK_PERIOD = 10;
always #(CLK_PERIOD/2) clk=~clk;

initial begin
    $dumpfile("tb/tb.vcd");
	
	for (i = 0; i < `RN; i = i + 1)
	begin
		memory[i][0] = 0;
		memory[i][1] = 0;
	end
	
	for (i = 0; i < `RN; i = i + 1)
	begin
		core_availability_signals_out[i] = 1;
	end
	
	for (i = 0; i < `RN; i = i + 1)
	begin
		for (j = 0; j < `RN; j = j + 1)
		begin
			@(posedge clk)
			begin
				core_NO <= i;
				packet <= {1'b1, j[`CS-1:0], {(`PL-1-`CS){1'b0}}};
			end
			@(posedge clk)
			begin
				packet <= {1'b1, j[`CS-1:0], {(`PL-1-`CS){1'b1}}};
			end
			@(posedge clk)
			begin
				packet <= 0;
			end
		end
		#1000;
		for (j = 0; j < `RN; j = j + 1)
		begin
			assert (memory[j][0] == {1'b1, j[`CS-1:0], {(`PL-1-`CS){1'b0}}}) else $error("Packet %b not to be seen", {1'b1, j[`CS-1:0], {(`PL-1-`CS){1'b0}}});
			assert (memory[j][1] == {1'b1, j[`CS-1:0], {(`PL-1-`CS){1'b1}}}) else $error("Packet %b not to be seen", {1'b1, j[`CS-1:0], {(`PL-1-`CS){1'b1}}});
			memory[j][0] = 0;
			memory[j][1] = 0;
		end
	end
	#200;
    $finish;

end

always @(*)
begin

	for (t = 0; t < `RN; t = t + 1)
	begin
		if (DUT.core_inputs[t] != 0)
			if (DUT.core_inputs[t][`PL-1] == 0)
				memory[t][0] = DUT.core_inputs[t];
			else
				memory[t][1] = DUT.core_inputs[t];
	end

end

endmodule
